@@ -1,17 +1,14 @@
 var EXPORTED_SYMBOLS = ["Connector"];
-if (typeof DEBUG == "undefined") const DEBUG = false;
+Components.utils.import("resource://indexdata/runtime/core.js");
+Components.utils.import("resource://indexdata/runtime/ConnectorTemplate.js");
 Components.utils.import('resource://indexdata/runtime/Task.js');
 Components.utils.import('resource://indexdata/util/xmlHelper.js');
 Components.utils.import('resource://indexdata/util/objHelper.js');
+Components.utils.import('resource://indexdata/util/helper.js');
 Components.utils.import("resource://indexdata/util/logging.js");
 var logger = logging.getLogger('');
-var contentFilter = Components.classes["@indexdata.com/contentfilter;1"].getService().wrappedJSObject;
-
-/*
- * This is a top level structure that represents the loaded
- * connector
- * @author jakub
- */
+var contentFilter = Components.classes["@indexdata.com/contentfilter;1"]
+                     .getService().wrappedJSObject;
 var Connector = function (template) {
   //hash of tasks indexed by name
   this.template = template || null;
@@ -26,19 +23,23 @@ var Connector = function (template) {
   //implementation of nsIWebProgress
   this.taskArgumentsUpdateCb = null;
   this.stepParametersUpdateCb = null;
-  this.properties = {};
+  this.properties = {'block_css': true,
+                     'block_images':true,
+                     'block_objects':true};
   this.data = {session:{}, system:{}};
   this.metaData = null;
   this.capabilityFlags = {}; // 'supports-fq' => false
     // indexed by flag name, value is true or false.
     // Only the flags explicitly set by the user, the rest come from defaults
-  this.inEngine = false;
   this.pageOverride = undefined;
   this.urlOverride = undefined;
 };
 
 Connector.prototype = {
+  
+  /////////////////////
   // page accessibility
+  
   getPageWindow: function () {
     var pageWindow = null;
     if (this.forcedPageWindow) {
@@ -49,19 +50,18 @@ Connector.prototype = {
       var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
       var mainWindow = wm.getMostRecentWindow("navigator:browser");
-      if (mainWindow)
-        pageWindow = mainWindow.gBrowser;
+      if (mainWindow) pageWindow = mainWindow.gBrowser;
     }
     if (pageWindow && (pageWindow.contentDocument || pageWindow.document))
       return pageWindow;
     else
       throw new Error('The page window is null or malformed (no content doc)');
   },
-
+  
   setPageWindow: function (win) {
     this.forcedPageWindow = win;
   },
-
+  
   getRealPageDoc: function () {
     var pW = this.getPageWindow();
     if (pW.contentDocument)
@@ -71,13 +71,12 @@ Connector.prototype = {
     else
       return null;
   },
-
+  
   getPageDoc: function () {
-    if (this.pageOverride)
-      return this.pageOverride;
+    if (this.pageOverride) return this.pageOverride;
     return this.getRealPageDoc();
   },
-
+  
   // Get the actual URL of the current page
   // This is tricky, there are many ways to do this, and they
   // have subtle differences in the builder and in the engine
@@ -105,57 +104,56 @@ Connector.prototype = {
   //     B: a:OK b:OK c:undef d:OK   E:  a:OK b:OK c:undef d:OK
   //   6 .defaultView.parent.document.URL
   //     B: a:OK b:OK c:undef d:OK   E:  a:file:/// b:file:/// c:file:/// d:file:///
-
   getRealPageUrl: function() {
-      /* Debug tests
-      var p = this.getPageDoc();
-      try { dump("url1: " + p.URL + "\n");
-      } catch(e) { dump("url1: "+e+"\n"); };
-      try { dump("url2: " + p.location + "\n");
-      } catch(e) { dump("url2: "+e+"\n"); };
-      try { dump("url3: " + p.location.href + "\n");
-      } catch(e) { dump("url3: "+e+"\n"); };
-      try { dump("url4: " + p.location.URL + "\n");
-      } catch(e) { dump("url4: "+e+"\n"); };
-      try { dump("url5: " + p.defaultView.document.URL + "\n");
-      } catch(e) { dump("url5: "+e+"\n"); };
-      try { dump("url6: " + p.defaultView.parent.document.URL + "\n");
-      } catch(e) { dump("url6: "+e+"\n"); };
-      */
-      var url="";
-      try {
-        var pg = this.getPageDoc();
-        if (pg) // no page, can't have a URL
-            url = pg.location.href;
-      } catch(e) {};
-      if (url)
-          return "" + url ; // as a string
-      return "";
+    /* Debug tests
+    var p = this.getPageDoc();
+    try { dump("url1: " + p.URL + "\n");
+    } catch(e) { dump("url1: "+e+"\n"); };
+    try { dump("url2: " + p.location + "\n");
+    } catch(e) { dump("url2: "+e+"\n"); };
+    try { dump("url3: " + p.location.href + "\n");
+    } catch(e) { dump("url3: "+e+"\n"); };
+    try { dump("url4: " + p.location.URL + "\n");
+    } catch(e) { dump("url4: "+e+"\n"); };
+    try { dump("url5: " + p.defaultView.document.URL + "\n");
+    } catch(e) { dump("url5: "+e+"\n"); };
+    try { dump("url6: " + p.defaultView.parent.document.URL + "\n");
+    } catch(e) { dump("url6: "+e+"\n"); };
+    */
+    var url="";
+    try {
+      // can't have a URL without a page
+      var pg = this.getPageDoc();
+      if (pg) url = pg.location.href;
+    } catch(e) {};
+    if (url) return "" + url ; // "" + casts to string
+    return "";
   },
-
+  
   getPageUrl: function() {
-      if (this.urlOverride)
-          return this.urlOverride;
-      return this.getRealPageUrl();
+    if (this.urlOverride) return this.urlOverride;
+    return this.getRealPageUrl();
   },
-
+  
   // Set the page override dom tree, and its url, for future use.
   setPageOverride: function( dom, url ) {
-      this.pageOverride = dom;
-      if (url)
-          this.urlOverride = url;
+    this.pageOverride = dom;
+    if (url) this.urlOverride = url;
   },
-
+  
   clearPageOverride: function( ) {
-      this.pageOverride = undefined;
-      this.urlOverride = undefined;
+    this.pageOverride = undefined;
+    this.urlOverride = undefined;
   },
 
+
+  /////////////////////
+  // Tasks and templates
+  
   getTemplate: function () {
     return this.template;
   },
 
-  //Task handling
   findTask: function (taskName, args) {
     var taskList = this.tasks[taskName];
     if (!taskList) return null;
@@ -172,10 +170,8 @@ Connector.prototype = {
       }
       return t0;
     }
-
     for (var t=0; t<taskList.length; t++) {
-      if (typeof args == "undefined")
-        return taskList[0];
+      if (typeof args == "undefined") return taskList[0];
       var hasAllArgs = true;
       var argList = taskList[t].getUsedArgs();
       for (var a=0; a<args.length; a++) {
@@ -184,56 +180,52 @@ Connector.prototype = {
           break;
         }
       }
-      if (hasAllArgs)
-        return taskList[t];
+      if (hasAllArgs) return taskList[t];
     }
     // Dirty trick to get around the need of lots of dummy transforms
     // in the init task. See bug 4973.
     // If no matching task was found above, and if we are looking for
     // the init task, return the first (and probably only) one.
-    if ( taskName == "init" )
-      return taskList[0];
+    if ( taskName == "init" ) return taskList[0];
     // TODO - This should be removed in some distant version, when we no
     // longer have connector files without the polymorphic flag in their
     // init tasks. Leave it it for now (version 2.9) to preserve old
     // behavior.
     return null;
   },
-
+  
   findTasks: function (taskName) {
     return typeof this.tasks[taskName] != "undefined"
       ? this.tasks[taskName]
       : null;
   },
-
+  
   getTaskById: function (id) {
     return this.tasksById[id];
   },
-
+  
   getTasks: function () {
     //temporary, before the onArgUpd works
     this.taskList.sort( function (a,b) { return a.compareTo(b); } );
     return this.taskList;
   },
-
+  
   setTaskArgumentsUpdateCb: function(cb) {
     this.taskArgumentsUpdateCb = cb;
   },
-
+  
   onTaskArgumentsUpdate: function() {
-    if (this.taskArgumentsUpdateCb)
-      this.taskArgumentsUpdateCb();
+    if (this.taskArgumentsUpdateCb) this.taskArgumentsUpdateCb();
   },
-
+  
   setStepParametersUpdateCb: function(cb) {
     this.stepParametersUpdateCb = cb;
   },
-
+  
   onStepParametersUpdate: function(step) {
-    if (this.stepParametersUpdateCb)
-      this.stepParametersUpdateCb(step);
+    if (this.stepParametersUpdateCb) this.stepParametersUpdateCb(step);
   },
-
+  
   //create new implementation of a given task by name
   createTask: function (taskName) {
     var task = new Task(this, taskName);
@@ -241,7 +233,7 @@ Connector.prototype = {
     this.addTask(task);
     return task;
   },
-
+  
   //insert and index task object
   addTask: function (task) {
     // new task group
@@ -254,90 +246,84 @@ Connector.prototype = {
     //push to flat list, sort by names
     this.taskList.push(task);
   },
-
+  
   detachTask: function (task) {
     task.connector = null;
-    function isAttached (element) {
-      return !(element === task);
-    }
+    function isAttached (element) { return !(element === task); }
     this.tasks[task.name] = this.tasks[task.name].filter(isAttached);
-    if (this.tasks[task.name].length == 0) delete this.tasks[task.name];
+    if (this.tasks[task.name].length === 0) delete this.tasks[task.name];
     this.taskList = this.taskList.filter(isAttached);
     delete this.tasksById[task.id];
   },
 
-  getPrefs: function () {
-    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
-  	.getService(Components.interfaces.nsIPrefService)
-  	.getBranch("indexdata.cf.");
-    return prefs;
-  },
-
+  //////////////////////
+  // Saving and loading
+  
+  // Slightly misnamed, returns the configuration XML, but does not actually
+  // save anywhere.
   saveConf: function () {
     this.recalculateCapabilityFlags(); // in case defaults have changed
-       // as can happen if the user edits steps
-    var connectorNode = xmlHelper.doc("connector");
+                                       // as can happen if the user edits steps
+    // connector node
+    var cn = xmlHelper.doc("connector");
+    if (this.template && this.template.name) {
+      cn.setAttribute('template', this.template.name);
+    }
+    if (core.version) cn.setAttribute('version', core.version);
     // meta data
-    var metaDataNode = null;
-
-    // store the cf builder version in meta data
-    if (this.builderVersion) this.metaData.cfbuilder_version = this.builderVersion;
-
+    var mdn = null;
     for (var name in this.metaData) {
-      if (metaDataNode == null) {
-        metaDataNode = xmlHelper.appendNode(connectorNode, "metaData");
-      }
+      if (mdn === null) mdn = xmlHelper.appendNode(cn, "metaData");
       if (this.metaData[name]) {
-        xmlHelper.appendNode(metaDataNode, "meta", null,
+        xmlHelper.appendNode(mdn, "meta", null,
         {"name" : name, "content" : this.metaData[name]});
       }
     }
-
     // Capability flags
-    var flagsNode = null;
+    var fn = null;
     for ( var flag in this.capabilityFlags ) {
-        if ( this.capabilityFlags[flag] != undefined ) {
-            if ( flagsNode == null )
-                flagsNode = xmlHelper.appendNode(connectorNode, "CapabilityFlags");
-            xmlHelper.appendNode(flagsNode, "flag", null,
-                {"name" : flag, "value" : this.capabilityFlags[flag]});
-        }
-    }
-    // properties
-    objHelper.saveProperties(this.properties, connectorNode);
-
-    for (var t in this.tasks) {
-      for (var i=0; i<this.tasks[t].length; i++) {
-        var taskNode = xmlHelper.appendNode(connectorNode, "task", null,
-            {"name": this.tasks[t][i].getName()});
-        this.tasks[t][i].saveConf(taskNode);
+      if ( this.capabilityFlags[flag] != undefined ) {
+        if ( fn === null ) fn = xmlHelper.appendNode(cn, "CapabilityFlags");
+        xmlHelper.appendNode(fn, "flag", null,
+            {"name" : flag, "value" : this.capabilityFlags[flag]});
       }
     }
-    return connectorNode;
+    // properties
+    objHelper.saveProperties(this.properties, cn);
+    for (var t in this.tasks) {
+      for (var i=0; i<this.tasks[t].length; i++) {
+        var tn = xmlHelper.appendNode(cn, "task", null,
+            {"name": this.tasks[t][i].getName()});
+        this.tasks[t][i].saveConf(tn);
+      }
+    }
+    return cn;
   },
-
+  
   saveConfToFile: function (file) {
     xmlHelper.saveDoc(file, this.saveConf());
   },
-
+  
   // load from connector doc and session parameters
   loadConf: function (doc, session) {
+    core.lastconnector = ""; // forget what we had before
     if (doc.getElementsByTagName("connector").length !== 1 ||
         doc.firstChild.nodeName.toLowerCase() !== "connector")
       throw new Error("file is not a proper connector")
-
-    var connectorNode = doc.firstChild;
-
-    var inBuilder = false;
-    try {
-      var xulAppInfo = Components.classes["@mozilla.org/xre/app-info;1"]
-                            .getService(Components.interfaces.nsIXULAppInfo);
-      if (xulAppInfo.name=="Firefox")
-        inBuilder = true;
-    } catch (e) {
+    var cn = doc.firstChild;
+    if (cn.hasAttribute('template')) {
+      this.template = new ConnectorTemplate();
+      this.template.loadFromUUID(cn.getAttribute('template'));
     }
-
-    if (inBuilder) {
+    // we can't use a connector from a newer major version
+    if (cn.hasAttribute('version')) {
+      let loadVer = cn.getAttribute('version');
+      if (!helper.checkVersion(loadVer, core.version)) {
+        throw new Error('Connector generated by CF ' + loadVer
+          + 'with newer major version than current ' + core.version);
+      }
+    }
+    if (core.inBuilder) {
       // remove generated nodes
       var stripGen = function (node) {
         var children = node.childNodes;
@@ -347,54 +333,46 @@ Connector.prototype = {
               (children[i].textContent === "undefined")) {
             node.removeChild(children[i]);
           } else {
-            if (children[i].hasChildNodes()) {
-              stripGen(children[i]);
-            }
+            if (children[i].hasChildNodes()) stripGen(children[i]);
           }
         }
       }
-      stripGen(connectorNode);
+      stripGen(cn);
     }
-
     // load tasks
     this.tasks = {};
     this.taskList = [];
     this.taskById = {};
-    var taskNodes = connectorNode.getElementsByTagName("task");
-    for (var i=0; i<taskNodes.length; i++) {
-      var name = taskNodes[i].getAttribute("name");
+    var tns = cn.getElementsByTagName("task");
+    for (var i=0; i < tns.length; i++) {
+      var name = tns[i].getAttribute("name");
       var task = new Task(this, name);
-      task.loadConf(taskNodes[i]);
+      task.loadConf(tns[i]);
       this.addTask(task);
     }
-
     // Load capability flags
     this.capabilityFlags = {};
     //dump("Loading flags\n");
-    var capgrpNodes = connectorNode.getElementsByTagName("CapabilityFlags");
+    var capgrpNodes = cn.getElementsByTagName("CapabilityFlags");
     if ( capgrpNodes.length < 1 ) {
         //dump("No capability flags found, must be an old connector\n");
     } else {
-        if (capgrpNodes.length > 1)
-            dump("Multiple capabilityFlags found, using first...");
-        var capNodes = capgrpNodes[0].getElementsByTagName("flag");
-        for (var i=0; i<capNodes.length; i++) {
-            var flag = capNodes[i].getAttribute("name");
-            var val = capNodes[i].getAttribute("value");
-            if (val == "true" )
-                val = true;
-            else
-                val = false;
-            this.capabilityFlags[flag]=val;
-            //dump("Flag " + flag + " = " + val + "\n" );
-        }
+      if (capgrpNodes.length > 1)
+          dump("Multiple capabilityFlags found, using first...");
+      var capNodes = capgrpNodes[0].getElementsByTagName("flag");
+      for (var i=0; i<capNodes.length; i++) {
+        var flag = capNodes[i].getAttribute("name");
+        var val = capNodes[i].getAttribute("value");
+        if (val == "true" ) val = true;
+        else val = false;
+        this.capabilityFlags[flag]=val;
+        //dump("Flag " + flag + " = " + val + "\n" );
+      }
     }
-
     // load properties
-    this.properties = objHelper.loadProperties(connectorNode);
-
+    this.properties = objHelper.loadProperties(cn);
     this.metaData = {};
-    var mdNodes = connectorNode.getElementsByTagName("metaData");
+    var mdNodes = cn.getElementsByTagName("metaData");
     if (mdNodes.length < 1) {
       dump("Metadata node not found"+"\n");
     } else {
@@ -406,7 +384,6 @@ Connector.prototype = {
           = metaNodes[i].getAttribute("content");
       }
     }
-
     //missing md elems
     if (this.template) {
       for (var i=0; i<this.template.metaData.length; i++) {
@@ -415,18 +392,18 @@ Connector.prototype = {
           this.metaData[name] = "";
       }
     }
-
     this.updateContentFilter();
-
     // initialise session
-    logger.log(Level.DEBUG, "Connector session arguments: " + JSON.stringify(session) + "\n");
-    this.session = session || {};
+    logger.log(Level.DEBUG, "Connector session arguments: "
+               + JSON.stringify(session) + "\n");
+    this.data.session = session || {};
   },
-
+  
   loadConfFromFile: function(filePath, session) {
     this.loadConf(xmlHelper.openDoc2(filePath), session);
+    core.lastconnector = filePath;
   },
-
+  
   isComplete: function() {
     var taskTemplates = this.template.getTaskTemplates();
     for (var i=0; i<taskTemplates.length; i++) {
@@ -440,18 +417,7 @@ Connector.prototype = {
     // all tasks complete
     return true;
   },
-
-  // Tell the connector if we are inside engine or builder
-  // only the engine needs to set this to true, the flag defaults
-  // to false, which is OK for the builder.
-  setInEngineFlag: function( flag ) {
-      this.inEngine = flag;
-  },
-
-  insideEngine: function() {
-      return this.inEngine;
-  },
-
+  
   // configure content filter from connector properties
   updateContentFilter: function() {
     try {
@@ -468,34 +434,32 @@ Connector.prototype = {
       contentFilter.resetWhitelist();
       var bl = this.properties["blacklist"];
       var wl = this.properties["whitelist"];
-      if (bl)
-        for (var i=0; i<bl.length; i++)
-          contentFilter.addBlacklist(bl[i]);
-      if (wl)
-        for (var i=0; i<wl.length; i++)
-          contentFilter.addWhitelist(wl[i]);
-    } catch(e)  {
-      dump("Exception: "+e+"\n");
-    }
+      if (bl) {
+        for (var i=0; i<bl.length; i++) contentFilter.addBlacklist(bl[i]);
+      }
+      if (wl) {
+        for (var i=0; i<wl.length; i++) contentFilter.addWhitelist(wl[i]);
+      }
+    } catch(e) { dump("Exception: "+e+"\n"); }
   },
 
-  /////// Capability flags /////////
-
-
+  ////////////////////
+  // Capability flags
+  
   // Get the whole array of (explicitly set) capabilityFlags.
   getCapabilityFlags: function() {
-      return this.capabilityFlags;
+    return this.capabilityFlags;
   },
-
+  
   setCapabilityFlag: function(name,val) {
-      this.capabilityFlags[name]=val;
+    this.capabilityFlags[name]=val;
   },
-
+  
   // Clears all flags to undefined
   clearAllCapabilityFlags: function() {
-      this.capabilityFlags = [];
+    this.capabilityFlags = [];
   },
-
+  
   getCapabilityFlagDefault: function (name, defaultValue, alias) {
     for (var i=0; i<this.taskList.length; i++) {
       let block = this.taskList[i].block;
@@ -506,11 +470,10 @@ Connector.prototype = {
     }
     return defaultValue;
   },
-
+  
   // get an opinion for a  default value for a capability flag within a block
   // or null when no opinion is given
-  getCapabilityFlagDefaultBlock: function (block, name, alias) 
-  {
+  getCapabilityFlagDefaultBlock: function (block, name, alias) {
     for (let i=0; i<block.steps.length; i++) {
       let step = block.steps[i];
       if ((step.getClassName() !== "Comment") && !step.isDisabled ) {
@@ -521,8 +484,8 @@ Connector.prototype = {
         }
         if (alias) {
           let opinion = step.capabilityFlagDefault(alias);
-          logger.debug("Step "+step+" reported '"+opinion+"' on flag "+alias);
           if (opinion != undefined && opinion != null) {
+            logger.debug("Step "+step+" reported '"+opinion+"' on flag alias "+alias);
             return opinion;
           }
         }
@@ -541,7 +504,7 @@ Connector.prototype = {
     }
     return null;
   },
-
+  
   // Get the value of a capability flag, either from the array of
   // manually set flags, or by finding a default value for it from
   // the steps, or if anything else fails, using the given default-default.
@@ -551,7 +514,7 @@ Connector.prototype = {
       return this.capabilityFlags[name];
     return this.getCapabilityFlagDefault(name, defaultvalue, alias );
   },
-
+  
   // Recalculate the metadata element 'Flags' based on the flags set
   // before, and the defaults from the steps, and from the template.
   recalculateCapabilityFlags: function() {
@@ -589,4 +552,5 @@ Connector.prototype = {
       this.data.session = JSON.parse(this.properties.testSessionJSON);
     }
   },
+  
 };
