@@ -98,21 +98,25 @@ NavTo.prototype.draw = function(surface) {
 };
 
 NavTo.prototype.processConf = function () {
+  //this.task.debug("NavTo.processConf calculating usedArgs");
   let usedArgs = [];
   let url = this.conf.url;
   if (url) {
+    //this.task.debug("NavTo.processConf considering url '" + url + "'");
     // gather used arguments from inline JSONpaths
     let usedArgRegex = /\{\$\.input\.(.+?)\}/g;
     while ((match = usedArgRegex.exec(url)) !== null) {
       usedArgs.push(match[1]);
     }
   } else {
+    //this.task.debug("NavTo.processConf considering JSONPath " + JSON.stringify(this.conf.jp));
     // reading from a path instead
     if (this.conf.jp.path === "$.input") {
       usedArgs.push(this.conf.jp.key);
     }
   }
   this.conf.usedArgs = usedArgs;
+  //this.task.debug("NavTo.processConf: set usedArgs to " + JSON.stringify(this.conf.usedArgs));
 }
 
 NavTo.prototype.getUrl = function (task) {
@@ -135,7 +139,7 @@ NavTo.prototype.getUrl = function (task) {
 NavTo.prototype.getRawXml = function(task) {
     var url=this.getUrl(task);
     task.debug("Getting rawurl from " + url );
-    if ( this.task.connector.insideEngine() ) {
+    if (!core.inBuilder) {
         var dom = xmlHelper.fetchDoc( url, false, false );
         this.task.connector.setPageOverride(dom,url);
         task.debug("Got it");
@@ -269,3 +273,20 @@ NavTo.prototype.renderArgs = function () {
   else return this.conf.url;
 };
 
+
+NavTo.prototype.capabilityFlagDefault = function (flag) {
+    if (this.task.name === "init" && flag.substring(0, 5) == "init-") {
+	this.processConf(); // Refresh usedArgs
+	var usedArgs = this.conf.usedArgs;
+	//this.task.debug("nav_to init flag '" + flag + "' within init task: compare with " + usedArgs.length + " used args: " + JSON.stringify(usedArgs));
+	for (var i = 0; i < usedArgs.length; i++) {
+	    if (usedArgs[i] === flag.substring(5)) {
+		//this.task.debug("nav_to init flag '" + flag + "' true for arg " + i + " of " + usedArgs.length);
+		return true;
+	    }
+	}
+    }
+
+    // Pass up to the superclass's default implementation
+    return Step.prototype.capabilityFlagDefault.call(this, flag);
+}
