@@ -1,23 +1,21 @@
 var EXPORTED_SYMBOLS = ["app"];
-var xulHelper = null;
-
-// Only initialise the xulHelper object in the builder TODO: proper separation of app
+const APP_ID = "cfbuilder@indexdata.com";
+var Cc = Components.classes;
+var Ci = Components.interfaces;
+Components.utils.import("resource://indexdata/runtime/core.js");
 let xulAppInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
-if (xulAppInfo.name !=="ConnectorFramework") { /* BEGIN BUILDER ONLY WRAPPER */
-
+core.setPlatformVersion(xulAppInfo.platformVersion);
 Components.utils.import("resource://indexdata/util/Subject.js");
 Components.utils.import("resource://indexdata/runtime/ConnectorTemplate.js");
 Components.utils.import("resource://indexdata/runtime/Connector.js");
 Components.utils.import('resource://indexdata/util/Highlighter.js');
 Components.utils.import('resource://indexdata/util/logging.js');
 Components.utils.import("resource://gre/modules/AddonManager.jsm");
-
-const APP_ID = "cfbuilder@indexdata.com";
-
 var logger = logging.getLogger();
 var IS_EXTENSION = true;
-var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
-  .getService(Components.interfaces.nsIWindowMediator);
+var wm = Cc["@mozilla.org/appshell/window-mediator;1"]
+  .getService(Ci.nsIWindowMediator);
+var idprefs = core.idprefs;
 
 /**
  * This class contains all global vars used throuoghout the application
@@ -33,7 +31,6 @@ app = new Subject([
     "version",
     "mainWindow",
     "template",
-    "templateFile",
     "activeTest",
     "connector",
     "selectedTask",
@@ -45,30 +42,38 @@ app = new Subject([
     "listAllSteps",
     "highlighter",
     "quit",
-    "fullqueryEdited"
+    "fullqueryEdited",
+    "proxied"
   ]);
 
+// we wait for the addon object before loading anything
+app.subscribe('addon', function() {
+  if (!app.loadState()) app.openDefaultTemplate();
+});
+app.set('proxied', false);
 app.set("id", APP_ID);
 AddonManager.getAddonByID(app.id, function(addon) {
-  app.set("addon", addon);
+  core.set("cfRootFile", addon.getResourceURI('/').QueryInterface(Ci.nsIFileURL).file);
   try {
-    // prefer version from IDMETA if found (git checkout)
-    let file = addon.getResourceURI('/').QueryInterface(Components.interfaces.nsIFileURL).file;
-    if (file && file.parent) {
+    // prefer version from IDMETA if found (dereference cfroot)
+    if (core.cfRootFile.target) {
+      let file = Components.classes["@mozilla.org/file/local;1"]
+          .createInstance(Components.interfaces.nsILocalFile);
+      file.initWithPath(core.cfRootFile.target);
       file = file.parent;
       file.append('IDMETA');
       if (file.isFile()) {
-        let stream = Components.classes["@mozilla.org/network/file-input-stream;1"]
-          .createInstance(Components.interfaces.nsIFileInputStream);
+        let stream = Cc["@mozilla.org/network/file-input-stream;1"]
+          .createInstance(Ci.nsIFileInputStream);
         stream.init(file, 0x01, 0444, 0);
-        stream.QueryInterface(Components.interfaces.nsILineInputStream);
+        stream.QueryInterface(Ci.nsILineInputStream);
         let line = {};
         let hasmore = true;
         while (hasmore) {
           hasmore = stream.readLine(line);
           let m = line.value.match(/^VERSION=(.*)$/);
           if (m) {
-            app.set("version", m[1]);
+            core.set("version", m[1]);
           }
         }
         stream.close();
@@ -77,28 +82,26 @@ AddonManager.getAddonByID(app.id, function(addon) {
   } catch (e) {
   }
   // otherwise use version from install.rdf
-  if (!app.version) app.set("version", addon.version);
+  if (!core.version) core.set("version", addon.version);
+  app.set("addon", addon);
 });
 // Block so we know we have the addon when we load the auto-saved connector
-Components.classes["@mozilla.org/thread-manager;1"]
-  .getService(Components.interfaces.nsIThreadManager)
+Cc["@mozilla.org/thread-manager;1"]
+  .getService(Ci.nsIThreadManager)
   .currentThread.processNextEvent(false);
 
 app.saveTimer = null;
 app.listAllSteps = false;
 try {
-  let prefs = Components.classes["@mozilla.org/preferences-service;1"]
-    .getService(Components.interfaces.nsIPrefService)
-    .getBranch("indexdata.cf.");
-  app.stickyNewStep = prefs.getBoolPref("stickyNewStep", app.stickyNewStep);
+  app.stickyNewStep = idprefs.getBoolPref("stickyNewStep", app.stickyNewStep);
 } catch (e) { 
   app.stickyNewStep = false;
 }
 app.tabReuse = {};
 
 // Observer -- right now just for quit
-let observerService = Components.classes["@mozilla.org/observer-service;1"]
-  .getService(Components.interfaces.nsIObserverService);
+let observerService = Cc["@mozilla.org/observer-service;1"]
+  .getService(Ci.nsIObserverService);
 observerService.addObserver(app, "quit-application", false);
 
 app.observe = function (subject, topic, data) {
@@ -110,32 +113,19 @@ app.openResAndDo = function (path, onres) {
     throw new Error("path must be an array");
   if (typeof onres != "function")
     throw new Error("onres arg must be a function");
-  //FF 3, simulute async API
-  var em = Components.classes["@mozilla.org/extensions/manager;1"];
-  if (em) {
-    logger.debug("Found nsIExtensionManager..");
-    em = em.getService(Components.interfaces.nsIExtensionManager);
-    var file = em.getInstallLocation(APP_ID).getItemFile(APP_ID, "");
-    for (var i=0; i<path.length; i++) {
-      file.append(path[i]);
-    }
-    logger.debug("Opening resource at "+file.path);
+  Components.utils.import("resource://gre/modules/AddonManager.jsm");
+  AddonManager.getAddonByID(APP_ID, function(addon) {
+    var file = addon.getResourceURI(path.join('/'))
+    .QueryInterface(Ci.nsIFileURL).file;
+    logger.debug("Opening resource at "+file.path)
     onres(file);
-  } else {
-    Components.utils.import("resource://gre/modules/AddonManager.jsm");
-    AddonManager.getAddonByID(APP_ID, function(addon) {
-      var file = addon.getResourceURI(path.join('/'))
-      .QueryInterface(Components.interfaces.nsIFileURL).file;
-      logger.debug("Opening resource at "+file.path)
-      onres(file);
-    });
-  }
+  });
 };
 
 app.disableApp = function () {
-  var man = Components.classes["@mozilla.org/extensions/manager;1"];
+  var man = Cc["@mozilla.org/extensions/manager;1"];
   if (man) {
-    man = man.getService(Components.interfaces.nsIExtensionManager);
+    man = man.getService(Ci.nsIExtensionManager);
     man.disableItem(app.id);
   } else {
     Components.utils.import("resource://gre/modules/AddonManager.jsm");
@@ -204,20 +194,23 @@ app.openOrReuseTab = function (url) {
 };
 
 app.openDefaultTemplate = function () {
-  //builder pref, id branch
-  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
-    .getService(Components.interfaces.nsIPrefService)
-    .getBranch("indexdata.cf.");
-  var file = app.openResAndDo(['templates',
-      prefs.getCharPref('defaulttemplate')],
-      function (file) {
-        app.loadTemplate(file);
-      });
+  app.template = new ConnectorTemplate();
+  try {
+    app.template.loadFromUUID(idprefs.getCharPref('defaulttemplate'));
+  } catch (e) {
+    logger.debug(e);
+    // TODO: it should be possible to operate without a template and that 
+    // should be the fallback instead of hardcoding
+    app.popupError("Cannot open default template: " + e.message + " Loading fallback.");
+    app.template.loadFromUUID('search.cft');
+    idprefs.setCharPref('defaulttemplate', 'search.cft');
+    app.notify("template");
+    if (!app.connector) app.newConnector();
+  }
 };
 
 app.loadTemplate = function (file) {
   app.template = new ConnectorTemplate();
-  app.templateFile = file;
   logger.debug("Loading template from " + file.path);
   try {
     app.template.loadFromFile(file);
@@ -232,7 +225,7 @@ app.loadTemplate = function (file) {
 };
 
 app.popupError = function (errorMsg) {
-  app.mainWindow.alert(errorMsg);
+  if (app.mainWindow) app.mainWindow.alert(errorMsg);
 };
 
 app.saveFileAs = function (winCtx) {
@@ -257,7 +250,17 @@ app.saveFileAs = function (winCtx) {
       app.set("lastOpened", fp.file);
     }, {"name" : "CF Connector", "ext" : "*.cf"}, null, proposedName, 'cf');
 };
-
+app.changedConnector = function () {
+  // for the time being we coerce templateless connectors into the
+  // current template as per previous behaviour
+  if (!app.connector.template) app.connector.template = app.template;
+  // use test session values for builder
+  // * only on load so they persist between task invocations
+  app.connector.useSessionTestValues();
+  app.set("canSave", false);
+  app.notify("template");
+  app.notify("connector");
+};
 app.saveFile = function (windowCtx) {
   if (app.lastOpened == null) return app.saveFileAs(windowCtx);
   try {
@@ -271,54 +274,38 @@ app.saveFile = function (windowCtx) {
 };
 
 app.loadFile = function (windowContext) {
-  if (!app.promptUnsaved())
-    return false;
+  if (!app.promptUnsaved()) return false;
   app.openBrowseDialog(windowContext, "Choose connector file..",
     "load", function (fp) {
-      app.connector = new Connector(app.template);
-      if (app.version) app.connector.builderVersion = app.version;
+      app.connector = new Connector();
       try {
-        app.connector.loadConfFromFile(fp.file.path);
+        app.connector.loadConfFromFile(fp.file.path); 
       } catch (e) {
         logger.debug(e);
         app.popupError("Cannot open file: " + e.message);
         return;
       }
-      // use test session values for builder
-      // * only on load so they persist between task invocations
-      app.connector.useSessionTestValues();
-      var bWin = app.getBrowserWidget(windowContext);
       app.set("lastOpened", fp.file);
-      app.notify("connector");
-      app.set("canSave", false);
+      app.changedConnector();
     }, {"name" : "CF Connectors", "ext" : "*.cf"}, null, null, 'cf'
   );
 };
 
 app.newConnector = function () {
-  if (!app.promptUnsaved())
-    return false;
-  app.set("canSave", false);
+  if (!app.promptUnsaved()) return false;
   app.connector = new Connector(app.template);
-  if (app.version) app.connector.builderVersion = app.version;
-  app.lastOpened = null;
   for (var key in app.template.requirements) {
     logger.debug(" - creating task from requirements for '"+key+"'");
     var task = app.connector.createTask(key);
     task.createDefaultTests();
   }
-  app.connector.metaData = {};
-  for (var i=0; i<app.template.metaData.length; i++) {
-    var name = app.template.metaData[i].name;
-    app.connector.metaData[name] = "";
-  }
-  app.notify("connector");
-  app.notify("lastOpened");
+  app.changedConnector();
+  app.set('lastOpened', null);
 };
 
 app.promptUnsaved = function () {
-  var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-    .getService(Components.interfaces.nsIPromptService);
+  var prompts = Cc["@mozilla.org/embedcomp/prompt-service;1"]
+    .getService(Ci.nsIPromptService);
   if (!app.canSave || prompts.confirm(null, "Please confirm", "Discard unsaved changes?"))
     return true;
   else
@@ -326,8 +313,7 @@ app.promptUnsaved = function () {
 }
 
 app.changeTemplate = function (winCtx) {
-  if (!app.promptUnsaved())
-    return false;
+  if (!app.promptUnsaved()) return false;
   var initDir = null;
   app.openResAndDo(["templates"],
     function (initDir) {
@@ -344,8 +330,8 @@ app.changeTemplate = function (winCtx) {
 
 app.openBrowseDialog = function (winCtx, winName, mode, onSelect, ftype,
                       initDir, defaultFileName, defaultExtension) {
-  const nsIFilePicker = Components.interfaces.nsIFilePicker;
-  var fp = Components.classes["@mozilla.org/filepicker;1"]
+  const nsIFilePicker = Ci.nsIFilePicker;
+  var fp = Cc["@mozilla.org/filepicker;1"]
     .createInstance(nsIFilePicker);
   fp.init(winCtx, winName,
       mode === "load" ? nsIFilePicker.modeOpen : nsIFilePicker.modeSave);
@@ -368,8 +354,8 @@ app.openBrowseDialog = function (winCtx, winName, mode, onSelect, ftype,
           fp.file.path.indexOf(defaultExtension, fp.file.path.length -
             defaultExtension.length) == -1) {
         var prompts =
-        Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-          .getService(Components.interfaces.nsIPromptService);
+        Cc["@mozilla.org/embedcomp/prompt-service;1"]
+          .getService(Ci.nsIPromptService);
         confirmed = prompts.confirm(null, "Please confirm",
           "You are about to save the file without its default extension (."
           +defaultExtension+") and your platform does not append extensions "
@@ -389,62 +375,47 @@ app.openBrowseDialog = function (winCtx, winName, mode, onSelect, ftype,
 };
 
 app.saveState = function (timer) {
-  var xs = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"].
-    createInstance(Components.interfaces.nsIDOMSerializer);
-  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
-    .getService(Components.interfaces.nsIPrefService)
-    .getBranch("indexdata.cf.");
-  prefs.setCharPref("state.connector",
+  let xs = Cc["@mozilla.org/xmlextras/xmlserializer;1"].
+    createInstance(Ci.nsIDOMSerializer);
+  idprefs.setCharPref("state.connector",
     xs.serializeToString(app.connector.saveConf()));
-  prefs.setComplexValue("state.templatefile",
-    Components.interfaces.nsILocalFile, app.templateFile);
   if (app.lastOpened && app.lastOpened.path)
-    prefs.setComplexValue("state.lastopenedfile",
-      Components.interfaces.nsILocalFile, app.lastOpened);
+    idprefs.setComplexValue("state.lastopenedfile",
+      Ci.nsILocalFile, app.lastOpened);
   else
-    prefs.setCharPref("state.lastopenedfile", "");
-  prefs.setBoolPref("state.cansave", app.canSave);
+    idprefs.setCharPref("state.lastopenedfile", "");
+  idprefs.setBoolPref("state.cansave", app.canSave);
   app.saveTimer = null;
   logger.debug("Connector state saved.");
 };
-
 app.saveStateIfYouHaventRecently = function () {
   if (app.connector && app.saveTimer === null) {
-    app.saveTimer = Components.classes["@mozilla.org/timer;1"]
-          .createInstance(Components.interfaces.nsITimer);
+    app.saveTimer = Cc["@mozilla.org/timer;1"]
+          .createInstance(Ci.nsITimer);
     var timerCb = { notify: app.saveState};
     app.saveTimer.initWithCallback(timerCb, 2000,
-      Components.interfaces.nsITimer.TYPE_ONE_SHOT);
+      Ci.nsITimer.TYPE_ONE_SHOT);
   }
 };
-
 app.subscribe("connector", app.saveStateIfYouHaventRecently);
 app.subscribe("canSave", app.saveStateIfYouHaventRecently);
 app.subscribe("template", app.saveStateIfYouHaventRecently);
 app.subscribe("quit", app.saveState);
-
 app.loadState = function () {
-  var prefs = Components.classes["@mozilla.org/preferences-service;1"]
-    .getService(Components.interfaces.nsIPrefService)
-    .getBranch("indexdata.cf.");
   try {
-    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
-      .createInstance(Components.interfaces.nsIDOMParser);
-    var connectorString = prefs.getCharPref("state.connector")
-    var templateFile = prefs.getComplexValue("state.templatefile", Components.interfaces.nsILocalFile);
-    var lastOpened = prefs.getCharPref("state.lastopenedfile");
+    var parser = Cc["@mozilla.org/xmlextras/domparser;1"]
+      .createInstance(Ci.nsIDOMParser);
+    var connectorString = idprefs.getCharPref("state.connector")
+    var lastOpened = idprefs.getCharPref("state.lastopenedfile");
     if (lastOpened)
-      lastOpened = prefs.getComplexValue("state.lastopenedfile", Components.interfaces.nsILocalFile);
-    else
-      lastOpened = null;
-    var canSave = prefs.getBoolPref("state.cansave")
-
-    app.loadTemplate(templateFile);
+      lastOpened = idprefs.getComplexValue("state.lastopenedfile", Ci.nsILocalFile);
+    else lastOpened = null;
+    app.connector = new Connector();
     app.connector.loadConf(parser.parseFromString(connectorString, "text/xml"));
-    app.connector.useSessionTestValues();
-    app.set("canSave", canSave);
+    if (!app.template) app.openDefaultTemplate();
+    app.changedConnector();
+    app.set("canSave", idprefs.getBoolPref("state.cansave"));
     app.set("lastOpened", lastOpened);
-    app.notify("connector");
     return true;
   } catch (err) {
     logger.debug(err);
@@ -472,17 +443,17 @@ app.newHl = function (doc, cb) {
 // Set proxy to proxy from init, if available.
 // Remove proxy if set.
 app.toggleProxy = function (e) {
-  let prefs = Components.classes["@mozilla.org/preferences-service;1"]
-    .getService(Components.interfaces.nsIPrefService).getBranch("network.proxy.");
+  if (!app.proxyButton) app.proxyButton = e.target;
+  let prefs = Cc["@mozilla.org/preferences-service;1"]
+    .getService(Ci.nsIPrefService).getBranch("network.proxy.");
   if (prefs.prefHasUserValue("type")) var type = prefs.getIntPref("type");
-
-  // If proxy enabled, disable and update button.
-  if (type && type > 0) {
+  // If proxy enabled (by us), disable and update button.
+  if (app.proxied && type && type > 0) {
     prefs.setIntPref("type", "0");
-    e.target.setAttribute("label", "Proxy?");
+    app.proxyButton.setAttribute("label", "Proxy?");
+    app.set('proxied', false);
     return;
   }
-
   // Otherwise see if we can find an init task w/proxyip
   let ipRegex = /^([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})(?:\:([0-9]{1,5}))?$/;
   let initTasks = this.connector.findTasks('init');
@@ -495,37 +466,48 @@ app.toggleProxy = function (e) {
   if (!(Array.isArray(result) && (result.length > 1))) return;
   let ip = result[1];
   let port = result[2] || "80";
-  prefs.setCharPref("http", ip);
-  prefs.setIntPref("http_port", parseInt(port));
-  prefs.setIntPref("type", 1);
-  e.target.setAttribute("label", "Proxied.");
+  if (ip && port) {
+    prefs.setCharPref("http", ip);
+    prefs.setIntPref("http_port", parseInt(port));
+    prefs.setIntPref("type", 1);
+    app.proxyButton.setAttribute("label", "Proxied.");
+    app.set('proxied', true);
+  }
   return;
 };
+app.subscribe('quit', function () {
+  if (app.proxied) app.toggleProxy();
+});
+app.subscribe('connector', function () {
+  if (app.proxied) {
+    // It was on because CF toggled it on some previous connector.
+    app.toggleProxy();
+    // Since it was on because of a connector, we should try turning
+    // it back on. It will stay off if this connector doesn't have one.
+    app.toggleProxy();
+  }
+});
 
 // Window loading
-
 app.newStepWindow = function () {
   app.mainWindow.open('chrome://cfbuilder/content/step_browser.xul',
           'newStepWindow', 'chrome,width=240,height=512,resizable=yes').focus();
 };
-
 app.newScratchWindow = function () {
   app.mainWindow.open('chrome://cfbuilder/content/scratch.xul',
           'scratchWindow', 'chrome,width=420,height=600,resizable=yes').focus();
 };
-
 app.newMetaWindow = function () {
-  app.mainWindow.openDialog('chrome://cfbuilder/content/meta_editor.xul', 'metaEditor', 'chrome,resizable=yes').focus();
+  app.mainWindow.openDialog('chrome://cfbuilder/content/meta_editor.xul',
+          'metaEditor', 'chrome,resizable=yes').focus();
 };
-
 app.newTaskWindow = function () { app.mainWindow.open('chrome://cfbuilder/content/new_task.xul',
           'newTaskWindow', 'chrome,width=300,height=400,resizable=yes').focus();
 };
-
 app.newTestEditWindow = function () {
-  app.mainWindow.open('chrome://cfbuilder/content/test_editor.xul', 'testEditor', 'chrome,resizable=yes').focus();
+  app.mainWindow.open('chrome://cfbuilder/content/test_editor.xul',
+          'testEditor', 'chrome,resizable=yes').focus();
 };
-
 app.newJsonEditWindow = function (obj, cb, windowName) {
   let win = app.mainWindow.open('chrome://cfbuilder/content/jsoneditor/index.html',
     windowName, 'chrome, width=800, height=600,resizable=yes');
@@ -537,5 +519,4 @@ app.newJsonEditWindow = function (obj, cb, windowName) {
     cb(this.formatter.get());
   }, false);
 };
-
-} /* END BUILDER ONLY WRAPPER */
+app.init = function () {};
