@@ -34,12 +34,12 @@ var metaEditor = {
     this.inputs = [];
     this.flagsinput = null;
       // for the case this gets called from the capability flag editor
-    var metaData = this.getConnector().metaData;
+    var metaData = this.getConnector().metaData || {};
     // we ask the template what to draw
     var metaItems = this.getConnector().getTemplate().getMetaData();
     for (var i=0; i<metaItems.length; i++) {
       var name = metaItems[i].name;
-      var txtBoxId = "cf_" + name;
+      var metaElemId = "cf_" + name;
       var dispName = metaItems[i].displayName;
       if (!dispName)
         dispName = name.charAt(0).toUpperCase() 
@@ -47,43 +47,40 @@ var metaEditor = {
       var toolTip = metaItems[i].note;
       var row = xmlHelper.appendNode(rows, "row", null, { align: "center" });
       xmlHelper.appendNode(row, "label", null, {value: dispName, width: 120,
-        control: txtBoxId, crop: "end", tooltiptext: dispName+"\n\n"+toolTip});
-      var attrs = { value: metaData[name], userdata: name, 
-        flex: 1, size: 50, id: txtBoxId}
-      if (metaItems[i].long != undefined &&
-          metaItems[i].long == "yes") {
-        attrs.multiline = "true";
-        attrs.rows = 3;
+        control: metaElemId, crop: "end", 
+        tooltiptext: dispName+"\n\n"+toolTip});
+      var attrs = {
+        value: metaData[name] !== undefined ? metaData[name] : "",
+        userdata: name, 
+        flex: 1,
+        size: 50, id: metaElemId
       }
-      if (metaItems[i].editable != undefined &&
-          metaItems[i].editable == "false") {
-        attrs.readonly = "true";
+      if (metaItems[i].type === "boolean") { //checkbox
+        attrs.checked = attrs.value == true; //value attr will be ignored
+        var metaCtrl = xmlHelper.appendNode(row, "checkbox", null, attrs);
+      } else { //textbox
+        if (metaItems[i].long != undefined &&
+            metaItems[i].long == "yes") {
+          attrs.multiline = "true";
+          attrs.rows = 3;
+        }
+        if (metaItems[i].editable != undefined &&
+            metaItems[i].editable == "false") {
+          attrs.readonly = "true";
+        }
+        var metaCtrl = xmlHelper.appendNode(row, "textbox", null, attrs);
       }
-      var txtBox = xmlHelper.appendNode(row, "textbox", null, attrs);
-      this.inputs.push(txtBox);
+      this.inputs.push(metaCtrl);
       var win = this.win;
       if (metaItems[i].warn) {
-        txtBox.setAttribute("warnmsg", metaItems[i].warn);
-        txtBox.setAttribute("warnshown", "no");
-        txtBox.addEventListener("focus", function (e) {
+        metaCtrl.setAttribute("warnmsg", metaItems[i].warn);
+        metaCtrl.setAttribute("warnshown", "no");
+        metaCtrl.addEventListener("focus", function (e) {
           var show = this.getAttribute("warnshown") === "no";
           if (show) {
             var ret = win.confirm(this.getAttribute("warnmsg"));
             if (ret) this.setAttribute("warnshown", "yes");
           }
-          //tried dispatching custom event but after that the input is
-          //higlighted but has no cursor
-          /*if (e.detail === 999) {
-          } else {
-            logger.debug("Im not 999");
-            e.stopPropagation();
-            e.preventDefault();
-            win.alert(this.getAttribute("warnmsg"));
-            var myevt = win.document.createEvent("UIEvents");
-            myevt.initUIEvent("focus", true, true, win, 999);
-            this.dispatchEvent(myevt);
-          }
-          */
         }, false);
       }
     }
@@ -91,10 +88,14 @@ var metaEditor = {
   // Called when the user clicks OK. Nothing gets saved until then, so the
   // user can click cancel instead.
   saveMeta: function() {
+    let connector = this.getConnector();
+    if (typeof(connector.metaData) !== 'object') connector.metaData = {};
     for ( var i in this.inputs ) {
       var n = this.inputs[i].getAttribute("userdata");
       var v = this.inputs[i].value;
-      this.getConnector().metaData[n]=v;
+      if (this.inputs[i].nodeName == "checkbox")
+        v = this.inputs[i].checked ? "1" : "0";
+      connector.metaData[n]=v;
       dump("Save input: " + i + ": " + n + "='" + v + "'\n");
     }
     app.canSave = true;
