@@ -57,6 +57,21 @@ var jsonPathHelper = {
     return true;
   return null;
   },
+  // This extends the above to apply to arrays of conditions. 
+  conditionsCapable: function (conditions, flag, path, flagprefix) {
+    if (Array.isArray(conditions)) {
+      for (let i = 0; i < conditions.length; i++) {
+        if (typeof conditions[i].operands === 'object'
+          && Array.isArray(conditions[i].operands)) {
+          let ops = conditions[i].operands;
+          for (let j = 0; j < ops.length; j++) {
+            if (typeof ops[j].jp === 'object' && jsonPathHelper.capabilityFlagDefault(
+              ops[j].jp, flag, "$.input", "index")) { return true; }
+          }
+        }
+      }
+    }
+  },
 
 
   // Return the first element from an array matching a JSONpath string.
@@ -427,7 +442,7 @@ var jsonPathHelper = {
     let newSpec = JSON.parse(JSON.stringify(nodeSpec));
     newSpec.xpath = this.inlineReplace(newSpec.xpath, container);
     if (Array.isArray(newSpec.frames)) {
-      for (i = 0; i < newSpec.frames.length; i++) {
+      for (let i = 0; i < newSpec.frames.length; i++) {
         newSpec.frames[i] = this.inlineReplace(newSpec.frames[i], container);
       }
     }
