@@ -42,13 +42,21 @@ ConnectorRuntime.prototype = {
     aliasFile.initWithPath(modulePath);
     var aliasURI = ioService.newFileURI(aliasFile);
     resProt.setSubstitution("indexdata", aliasURI);
-    //load the required module
+    //set up core metainfo
+    Components.utils.import('resource://indexdata/runtime/core.js');
+    core.set('inBuilder', false);
+    let info = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo);
+    core.set('version', info.version);
+    core.setPlatformVersion(info.platformVersion);
+    //load the required modules
     Components.utils.import('resource://indexdata/runtime/Connector.js');
     Components.utils.import('resource://indexdata/runtime/Tester.js');
     Components.utils.import('resource://indexdata/util/logging.js');
+    // TODO: the cf root should be passed into init() explicitly rather than 
+    // assumed to be relative to the module path
+    core.set('cfRootFile', aliasFile.parent);
     //instantiate the connector
     this.connector = new Connector();
-    this.connector.setInEngineFlag(true);
     var appender = new CFAppender(logInterface);
     logging.modifyLoggers([appender]);
     this.logger = logging.getLogger();
